<?php
namespace Ak\Filter;

use Laminas\Filter\AbstractFilter;
use Laminas\Filter\FilterInterface;
use RuntimeException;

/**
 * Filtrira svaki element arraya koristeći neki drugi filter.
 *
 * @package Ak\Filter
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/filter
 */
final class ArrayFilter extends AbstractFilter implements FilterManagerAwareInterface
{
    use FilterManagerAwareTrait;

    /** @var FilterInterface */
    private $filter;

    /**
     * Konstruktor.
     *
     * @param FilterInterface|array $filterOrOptions
     */
    public function __construct($filterOrOptions = null)
    {
        if ($filterOrOptions instanceof FilterInterface) {
            $this->filter = $filterOrOptions;
        } else if ($filterOrOptions !== null) {
            $this->setOptions($filterOrOptions);
        }
    }

    /**
     * @return FilterInterface
     */
    public function getFilter(): ?FilterInterface
    {
        if (is_string($this->filter)) {
            $plugins = $this->getFilterPluginManager();
            $this->filter = $plugins->get($this->filter);
        }

        if (is_array($this->filter)) {
            $name = $this->filter['name'] ?? null;

            if (empty($name)) {
                throw new RuntimeException(
                    "Filter mora imatu ključ 'name'."
                );
            }

            $plugins = $this->getFilterPluginManager();
            $this->filter = $plugins->get($name, $this->filter['options'] ?? []);
        }

        if (!$this->filter instanceof FilterInterface) {
            throw new RuntimeException(sprintf(
                'Filter mora implementirati %s, a dobio sam %s.',
                FilterInterface::class,
                is_object($this->filter) ? get_class($this->filter) : gettype($this->filter)
            ));
        }

        return $this->filter;
    }

    /**
     * @param FilterInterface|array|string $filter
     */
    public function setFilter($filter): void
    {
        $this->filter = $filter;
    }

    /**
     * Returns the result of filtering $value
     *
     * @param  array $value
     * @throws \Laminas\Filter\Exception\RuntimeException If filtering $value is impossible
     * @return array
     */
    public function filter($value): ?array
    {
        if ($value === null) {
            return null;
        }

        $filter = $this->getFilter();

        if (!is_array($value)) {
            throw new \Laminas\Filter\Exception\RuntimeException(sprintf(
                "Vrijednost mora biti array, a dobio sam %s.",
                is_object($value) ? get_class($value) : gettype($value)
            ));
        }

        foreach ($value as &$item) {
            $item = $filter->filter($item);
        }

        return $value;
    }
}
